@@ -293,6 +293,7 @@ renv_dependencies_callback <- function(path) {
   cbname <- list(
     ".Rprofile"     = function(path) renv_dependencies_discover_r(path),
     "DESCRIPTION"   = function(path) renv_dependencies_discover_description(path),
+    "NAMESPACE"     = function(path) renv_dependencies_discover_namespace(path),
     "_bookdown.yml" = function(path) renv_dependencies_discover_bookdown(path),
     "_pkgdown.yml"  = function(path) renv_dependencies_discover_pkgdown(path),
     "_quarto.yml"   = function(path) renv_dependencies_discover_quarto(path),
@@ -434,7 +435,7 @@ renv_dependencies_find_dir_children <- function(path, root, depth) {
 
   # remove hard-coded ignores
   # (only keep DESCRIPTION files at the top level)
-  ignored <- c("packrat", "renv", "revdep", "vendor", if (depth) "DESCRIPTION")
+  ignored <- c("packrat", "renv", "revdep", "vendor", if (depth) c("DESCRIPTION", "NAMESPACE"))
   children <- children[!basename(children) %in% ignored]
 
   # compute exclusions
@@ -589,6 +590,37 @@ renv_dependencies_discover_description <- function(path,
 
 }
 
+renv_dependencies_discover_namespace <- function(path) {
+
+  tryCatch(
+    renv_dependencies_discover_namespace_impl(path),
+    error = warnify
+  )
+
+}
+
+renv_dependencies_discover_namespace_impl <- function(path) {
+
+  # parseNamespaceFile() expects to be called on an installed package,
+  # so we have to pretend our best here
+  library <- dirname(dirname(path))
+  package <- basename(dirname(path))
+  info <- parseNamespaceFile(
+    package     = package,
+    package.lib = library,
+    mustExist   = TRUE
+  )
+
+  # read package names from imports
+  packages <- map_chr(info$imports, `[[`, 1L)
+
+  renv_dependencies_list(
+    source   = path,
+    packages = sort(unique(packages))
+  )
+
+}
+
 renv_dependencies_discover_description_impl <- function(dcf, field, path) {
 
   # read field
@@ -610,6 +642,14 @@ renv_dependencies_discover_description_impl <- function(dcf, field, path) {
 
   m <- regexec(pattern, x)
   matches <- regmatches(x, m)
+  if (empty(matches))
+    return(list())
+
+  # drop R (https://github.com/rstudio/renv/issues/1806)
+  matches <- filter(matches, function(match) {
+    !identical(match[[2L]], "R")
+  })
+
   if (empty(matches))
     return(list())
 
@@ -719,7 +759,7 @@ renv_dependencies_discover_rmd_yaml_header <- function(path, mode) {
     values <- c(names(node), if (pstring(node)) node)
     for (value in values) {
       call <- tryCatch(parse(text = value)[[1]], error = function(err) NULL)
-      if (renv_call_matches(call, name = c("::", ":::"), n_args = 2)) {
+      if (renv_call_matches(call, name = c("::", ":::"), nargs = 2L)) {
         deps$push(as.character(call[[2L]]))
       }
     }
@@ -782,7 +822,7 @@ renv_dependencies_discover_chunks_ignore <- function(chunk) {
 
   # skip non-R chunks
   engine <- chunk$params[["engine"]]
-  ok <- is.character(engine) && engine %in% c("r", "rscript")
+  ok <- is.character(engine) && tolower(engine) %in% c("r", "rscript")
   if (!ok)
     return(TRUE)
 
@@ -873,7 +913,7 @@ renv_dependencies_discover_chunks <- function(path, mode) {
   if (mode %in% "qmd") {
     for (chunk in chunks) {
       engine <- chunk$params[["engine"]]
-      if (is.character(engine) && engine %in% c("r", "rscript")) {
+      if (is.character(engine) && tolower(engine) %in% c("r", "rscript")) {
         qdeps <- renv_dependencies_list(path, "rmarkdown")
         break
       }
@@ -1020,7 +1060,10 @@ renv_dependencies_discover_r <- function(path = NULL,
     renv_dependencies_discover_r_box,
     renv_dependencies_discover_r_targets,
     renv_dependencies_discover_r_glue,
+    renv_dependencies_discover_r_ggplot2,
     renv_dependencies_discover_r_parsnip,
+    renv_dependencies_discover_r_testthat,
+    renv_dependencies_discover_r_knitr,
     renv_dependencies_discover_r_database
   )
 
@@ -1028,12 +1071,12 @@ renv_dependencies_discover_r <- function(path = NULL,
   recurse(expr, function(node, stack) {
 
     # normalize calls (handle magrittr pipes)
-    node <- renv_call_normalize(node, stack)
+    if (is.call(node))
+      node <- renv_call_normalize(node, stack)
 
     # invoke methods on call objects
-    if (is.call(node))
-      for (method in methods)
-        method(node, stack, envir)
+    for (method in methods)
+      method(node, stack, envir)
 
     # return node
     node
@@ -1140,8 +1183,7 @@ renv_dependencies_discover_r_require_namespace <- function(node, stack, envir) {
 
 renv_dependencies_discover_r_colon <- function(node, stack, envir) {
 
-  ok <- renv_call_matches(node, name = c("::", ":::"), n_args = 2)
-
+  ok <- renv_call_matches(node, name = c("::", ":::"), nargs = 2L)
   if (!ok)
     return(FALSE)
 
@@ -1211,9 +1253,11 @@ renv_dependencies_discover_r_pacman <- function(node, stack, envir) {
 
 renv_dependencies_discover_r_modules <- function(node, stack, envir) {
 
-  # check for call of the form 'pkg::foo(a, b, c)'
-  colon <- renv_call_matches(node[[1]], name = c("::", ":::"), n_args = 2)
+  if (!is.call(node))
+    return(FALSE)
 
+  # check for call of the form 'pkg::foo(a, b, c)'
+  colon <- renv_call_matches(node[[1L]], name = c("::", ":::"), nargs = 2L)
   node <- renv_call_expect(node, "modules", c("import"))
   if (is.null(node))
     return(FALSE)
@@ -1257,6 +1301,14 @@ renv_dependencies_discover_r_modules <- function(node, stack, envir) {
 
 renv_dependencies_discover_r_import <- function(node, stack, envir) {
 
+  if (!is.call(node))
+    return(FALSE)
+
+  # require that usages are colon-prefixed
+  colon <- renv_call_matches(node[[1L]], name = c("::", ":::"), nargs = 2L)
+  if (!colon)
+    return(FALSE)
+
   node <- renv_call_expect(node, "import", c("from", "here", "into"))
   if (is.null(node))
     return(FALSE)
@@ -1274,16 +1326,22 @@ renv_dependencies_discover_r_import <- function(node, stack, envir) {
 
   # the '.from' argument is the package name, either a character vector of length one or a symbol
   from <- matched$.from
-  if (is.symbol(from))
-    from <- as.character(from)
-
-  ok <-
-    is.character(from) &&
-    length(from) == 1
+  if (is.symbol(from)) {
+    co <- node[[".character_only"]]
+    if (!identical(co, TRUE))
+      from <- as.character(from)
+  }
 
+  ok <- is.character(from) && length(from) == 1L
   if (!ok)
     return(FALSE)
 
+  # '.from' can also be an R script; if it appears to be a path, then ignore it
+  # https://github.com/rstudio/renv/issues/1743
+  if (grepl("\\.[rR]$", from, perl = TRUE) &&
+      grepl("[/\\]", from))
+    return(FALSE)
+
   envir[[from]] <- TRUE
   TRUE
 
@@ -1373,6 +1431,84 @@ renv_dependencies_discover_r_glue <- function(node, stack, envir) {
 
 }
 
+renv_dependencies_discover_r_ggplot2 <- function(node, stack, envir) {
+
+  node <- renv_call_expect(node, "ggplot2", "ggsave")
+  if (is.null(node))
+    return(FALSE)
+
+  # check for attempts to save to '.svg', and assume svglite is
+  # required in this scenario.
+  matched <- catch(match.call(function(filename, ...) {}, node))
+  if (inherits(matched, "error"))
+    return(FALSE)
+
+  filename <- matched$filename
+  if (!is.character(filename))
+    return(FALSE)
+
+  if (!endswith(filename, ".svg"))
+    return(FALSE)
+
+  envir[["svglite"]] <- TRUE
+  TRUE
+
+}
+
+renv_dependencies_discover_r_testthat <- function(node, stack, envir) {
+
+  # check for construction of JunitReporter
+  if (identical(node, call("$", as.symbol("JunitReporter"), as.symbol("new")))) {
+    envir[["xml2"]] <- TRUE
+    return(TRUE)
+  }
+
+  # check for calls to various test runners, which accept a reporter
+  node <- renv_call_expect(node, "testthat", c("test_package", "test_dir", "test_file"))
+  if (is.null(node))
+    return(FALSE)
+
+  candidates <- list(
+    "Junit",
+    "junit",
+    call("::", "testthat", "JunitReporter"),
+    as.symbol("JunitReporter")
+  )
+
+  reporter <- node$reporter
+  if (!is.null(reporter)) {
+    for (candidate in candidates) {
+      if (identical(candidate, reporter)) {
+        envir[["xml2"]] <- TRUE
+        return(TRUE)
+      }
+    }
+  }
+
+  FALSE
+
+}
+
+renv_dependencies_discover_r_knitr <- function(node, stack, envir) {
+  
+  matched <- is.call(node) && (
+    identical(node[[1L]], quote(knitr::opts_chunk$set)) ||
+    identical(node[[1L]], quote(opts_chunk$set))
+  )
+  
+  if (!matched)
+    return(FALSE)
+  
+  args <- as.list(node)
+  if (identical(args[["dev"]], "ragg_png")) {
+    envir[["ragg"]] <- TRUE
+    return(TRUE)
+  }
+  
+  FALSE
+  
+}
+
 renv_dependencies_discover_r_glue_impl <- function(string, node, envir) {
 
   # get open, close delimiters
@@ -1544,19 +1680,27 @@ renv_dependencies_discover_r_database <- function(node, stack, envir) {
 
   found <- FALSE
 
+  matched <- function(requirements) {
+    for (requirement in requirements)
+      envir[[requirement]] <<- TRUE
+    found <<- TRUE
+  }
+
   db <- renv_dependencies_database()
   enumerate(db, function(package, dependencies) {
     enumerate(dependencies, function(method, requirements) {
 
-      expect <- renv_call_expect(node, package, method)
-      if (is.null(expect))
-        return(FALSE)
+      if (is.call(node)) {
+        expect <- renv_call_expect(node, package, method)
+        if (!is.null(expect))
+          return(matched(requirements))
+      }
 
-      for (requirement in requirements)
-        envir[[requirement]] <- TRUE
-
-      found <<- TRUE
-      TRUE
+      if (is.symbol(node)) {
+        value <- as.character(node)
+        if (identical(value, method))
+          return(matched(requirements))
+      }
 
     })
   })
@@ -1575,6 +1719,7 @@ renv_dependencies_database <- function() {
 renv_dependencies_database_impl <- function() {
   db <- getOption("renv.dependencies.database", default = list())
   db$ggplot2$geom_hex <- "hexbin"
+  db$testthat$JunitReporter <- "xml2"
   db
 }
 
