@@ -5,7 +5,7 @@ renv_hash_text <- function(text) {
 
 renv_hash_description <- function(path) {
   filebacked(
-    scope    = "renv_hash_description",
+    context  = "renv_hash_description",
     path     = path,
     callback = renv_hash_description_impl
   )
@@ -32,10 +32,7 @@ renv_hash_description_impl <- function(path) {
   subsetted <- dcf[renv_vector_intersect(c(fields, remotes), names(dcf))]
 
   # sort names (use C locale to ensure consistent ordering)
-  ordered <- local({
-    renv_scope_locale("LC_COLLATE", "C")
-    subsetted[sort(names(subsetted))]
-  })
+  ordered <- subsetted[csort(names(subsetted))]
 
   # write to tempfile (use binary connection to ensure unix-style
   # newlines for cross-platform hash stability)
@@ -77,13 +74,11 @@ renv_hash_description_impl <- function(path) {
 
 renv_hash_description_remotes <- function(dcf) {
 
-  type <- dcf[["RemoteType"]]
-  if (is.null(type))
-    return(character())
-
-  if (type == "standard")
+  # ignore other remote fields for cranlike remotes
+  if (renv_record_cranlike(dcf))
     return(character())
 
+  # otherwise, include any other discovered remote fields
   grep("^Remote", names(dcf), value = TRUE)
 
 }
