@@ -57,7 +57,7 @@ renv_lockfile_read_preflight <- function(contents) {
 
     all <- unlist(parts, recursive = TRUE, use.names = FALSE)
 
-    renv_pretty_print(
+    caution_bullets(
       "The lockfile contains one or more merge conflict markers:",
       head(all, n = -1L),
       "You will need to resolve these merge conflicts before the file can be read."
