@@ -81,12 +81,14 @@ renv_ppm_transform_impl <- function(url) {
     return(binurl)
   }
 
-  # try to query the status endpoint
-  # TODO: this could fail if the URL is a proxy back to PPM?
+  # try to query the status endpoint -- if this fails, use our best guess?
   base <- dirname(dirname(url))
   status <- catch(renv_ppm_status(base))
-  if (inherits(status, "error"))
-    return(url)
+  if (inherits(status, "error")) {
+    parts <- c(dirname(url), "__linux__", platform, basename(url))
+    binurl <- paste(parts, collapse = "/")
+    return(binurl)
+  }
 
   # iterate through distros and check for a match
   for (distro in status$distros) {
@@ -166,10 +168,15 @@ renv_ppm_platform_impl <- function(file = "/etc/os-release") {
     id <- properties$ID %||% ""
 
     case(
-      identical(id, "ubuntu") ~ renv_ppm_platform_ubuntu(properties),
-      identical(id, "centos") ~ renv_ppm_platform_centos(properties),
-      identical(id, "rhel")   ~ renv_ppm_platform_rhel(properties),
-      grepl("\\bsuse\\b", id) ~ renv_ppm_platform_suse(properties)
+      identical(id, "ubuntu")    ~ renv_ppm_platform_ubuntu(properties),
+      identical(id, "centos")    ~ renv_ppm_platform_centos(properties),
+      identical(id, "rhel")      ~ renv_ppm_platform_rhel(properties),
+      identical(id, "rocky")     ~ renv_ppm_platform_rocky(properties),
+      identical(id, "almalinux") ~ renv_ppm_platform_alma(properties),
+      grepl("suse\\b", id)       ~ renv_ppm_platform_suse(properties),
+      identical(id, "sles")      ~ renv_ppm_platform_sles(properties),
+      identical(id, "debian")    ~ renv_ppm_platform_debian(properties),
+      identical(id, "amzn")      ~ renv_ppm_platform_amzn(properties)
     )
 
   }
@@ -202,10 +209,32 @@ renv_ppm_platform_rhel <- function(properties) {
   if (is.null(id))
     return(NULL)
 
-  paste0("centos", substring(id, 1L, 1L))
+  version <- ifelse(numeric_version(id) < "9", "centos", "rhel")
+  paste0(version, substring(id, 1L, 1L))
 
 }
 
+renv_ppm_platform_rocky <- function(properties) {
+
+  id <- properties$VERSION_ID
+  if (is.null(id))
+    return(NULL)
+
+  version <- ifelse(numeric_version(id) < "9", "centos", "rhel")
+  paste0(version, substring(id, 1L, 1L))
+
+}
+
+renv_ppm_platform_alma <- function(properties) {
+
+  id <- properties$VERSION_ID
+  if (is.null(id))
+    return(NULL)
+
+  version <- ifelse(numeric_version(id) < "9", "centos", "rhel")
+  paste0(version, substring(id, 1L, 1L))
+
+}
 
 renv_ppm_platform_suse <- function(properties) {
 
@@ -214,7 +243,41 @@ renv_ppm_platform_suse <- function(properties) {
     return(NULL)
 
   parts <- strsplit(id, ".", fixed = TRUE)[[1L]]
-  paste0("opensuse", parts[[1L]])
+  paste0("opensuse", parts[[1L]], parts[[2L]])
+
+}
+
+renv_ppm_platform_sles <- function(properties) {
+
+  id <- properties$VERSION_ID
+  if (is.null(id))
+    return(NULL)
+
+  parts <- strsplit(id, ".", fixed = TRUE)[[1L]]
+  paste0("opensuse", parts[[1L]], parts[[2L]])
+
+}
+
+renv_ppm_platform_debian <- function(properties) {
+
+  codename <- properties$VERSION_CODENAME
+  if (is.null(codename))
+    return(NULL)
+
+  codename
+
+}
+
+renv_ppm_platform_amzn <- function(properties) {
+
+  id <- properties$VERSION_ID
+  if (is.null(id))
+    return(NULL)
+
+  if (numeric_version(id) == "2")
+    return("centos7")
+
+  return(NULL)
 
 }
 
