@@ -45,6 +45,25 @@ renv_remotes_resolve <- function(spec, latest = FALSE) {
       return(record)
   }
 
+  # check for explicit local remotes
+  if (grepl("^local::", spec)) {
+    spec <- substring(spec, 8L)
+    record <- catch(renv_remotes_resolve_path(spec))
+    if (!inherits(record, "error"))
+      return(record)
+  }
+
+  # check for requests to install local packages -- note that depending on how
+  # the R package was built / generated, it's possible that it might not adhere
+  # to the "typical" R package names, so we try to be a bit flexible here
+  ext <- "(?:\\.tar\\.gz|\\.tgz|\\.zip)$"
+  if (grepl(ext, spec, perl = TRUE)) {
+    pathlike <- tryCatch(file.exists(spec), condition = identity)
+    if (identical(pathlike, TRUE)) {
+      return(renv_remotes_resolve_path(spec))
+    }
+  }
+
   # define error handler (tag error with extra context when possible)
   error <- function(e) {
 
@@ -432,6 +451,7 @@ renv_remotes_resolve_repository <- function(remote, latest) {
   if (latest && is.null(version)) {
     remote <- renv_available_packages_latest(package)
     version <- remote$Version
+    repository <- remote$Repository
   }
 
   list(
